/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "version.h"
#include "types.h"
#include "compiler.h"
#include "fwcmd.h"

#ifndef __CARL9170FW_CONFIG_H
#define __CARL9170FW_CONFIG_H

#define __CARL9170FW__

#define GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)

#if GCC_VERSION < 40400
# error "This firmware will not work if it is compiled with gcc versions < 4.4"
# error "See: http://gcc.gnu.org/gcc-4.4/changes.html / Caveats No. 4"
#endif

#if ((defined CONFIG_CARL9170FW_PRINTF) && 		\
     (!defined CONFIG_CARL9170FW_DEBUG_USB) &&		\
    (!defined CONFIG_CARL9170FW_DEBUG_UART))
# warning "You have disabled all debug message transports."
# warning "However CONFIG_CARL9170FW_PRINTF is still set..."
# warning "Which is a waste of firmware space, if you ask me."
#endif

#if ((defined CONFIG_CARL9170FW_WATCHDOG_BUTTON) &&	\
    !((defined CONFIG_CARL9170FW_GPIO_INTERRUPT) &&	\
      (defined CONFIG_CARL9170FW_WATCHDOG)))
# error "Watchdog Button does not work with your configuration."
#endif

#ifdef CONFIG_CARL9170FW_DEBUG_UART
# define CARL9170FW_UART_CLOCK_25M 0
#endif /* CONFIG_CARL9170FW_DEBUG_UART */

#if ((!defined CONFIG_CARL9170FW_WATCHDOG) && 		\
     (defined CONFIG_CARL9170FW_USB_WATCHDOG))
# error "USB Watchdog trigger only works if generic watchdog is also enabled."
#endif

#if ((!defined CONFIG_CARL9170FW_RADIO_FUNCTIONS) && 		\
     (defined CONFIG_CARL9170FW_PSM))
# error "Power save Management depends on CONFIG_CARL9170FW_RADIO_FUNCTION."
#endif

#ifdef CONFIG_CARL9170FW_HANDLE_BACK_REQ
#define CARL9170_BACK_REQS_NUM		4
#endif /* CONFIG_CARL9170FW_HANDLE_BACK_REQ */

#define CARL9170_TX_STATUS_NUM		(CARL9170_RSP_TX_STATUS_NUM)
#define CARL9170_INT_RQ_CACHES		16
#define AR9170_INT_MAGIC_HEADER_SIZE	12
#define CARL9170_USB_WATCHDOG_TRIGGER_THRESHOLD	4

#if ((defined CONFIG_CARL9170FW_DEBUG) ||	\
     (defined CONFIG_CARL9170FW_LOOPBACK))
#define CARL9170FW_UNUSABLE	y
#endif

static inline void __config_check(void)
{
	BUILD_BUG_ON(CARL9170_TX_STATUS_NUM % 2 || !CARL9170_TX_STATUS_NUM);
	BUILD_BUG_ON(CARL9170_USB_WATCHDOG_TRIGGER_THRESHOLD < 2);

#ifdef CONFIG_CARL9170FW_HANDLE_BACK_REQ
	BUILD_BUG_ON(!CARL9170_BACK_REQS_NUM);
#endif /* CONFIG_CARL9170FW_HANDLE_BACK_REQ */
}

#endif /* __CARL9170FW_CONFIG_H */
